void main() {
    float normalizedPosition = (v_path_distance / u_path_length);

    // feather_size is the size of the anti aliasing in x:length and y:width of the line...
    vec2 feather_size = vec2(lineFeather / u_path_length, lineFeather / lineWidth);

    // Evaluate helper variables for start cap and end cap...
    float sq_sin_w = sqrt(sin(v_tex_coord.y*3.141592653));
    float l_cap_radius = lineWidth / (u_path_length * 2.0);
    float l_cap_inner = l_cap_radius * sq_sin_w;
    float l_cap_outer = (l_cap_radius + feather_size.x) * sq_sin_w;

    // Prepare background color...
    vec4 bg_col = mix(startBackgroundColor, stopBackgroundColor, normalizedPosition);
    // Start cap...
    bg_col = mix(bg_col, startColor, smoothstep(1.0 - l_cap_outer, 1.0 - l_cap_inner, normalizedPosition));

    // Handle progress overlapping 100%...
    normalizedPosition += step(normalizedPosition, max(0.0, progress + l_cap_outer - 1.0));

    // Prepare colors...
    vec4 fg_col = mix(startColor, stopColor, normalizedPosition);
    vec4 glow_col = mix(fg_col, progressHeadGlowColor, progressHeadGlowIntensity);

    // End cap...
    bg_col = mix(bg_col, startColor, step(1.0, normalizedPosition));
    fg_col = mix(fg_col, glow_col, smoothstep(progress - l_cap_radius, progress + l_cap_radius, normalizedPosition));
    fg_col = mix(fg_col, bg_col, smoothstep(progress + l_cap_inner, progress + l_cap_outer, normalizedPosition));

    // Anti aliasing circle...
    float a = smoothstep(0.0, feather_size.y, 0.5-abs(v_tex_coord.y-0.5));
    fg_col.a *= a;
    
    // Premultiply alpha...
    fg_col.xyz *= fg_col.a;
    gl_FragColor = fg_col;
}
